@@ -69,7 +69,8 @@ struct hash_table_entry {
 	struct hash_table_entry *next;
 };
 
-int bytes = 0, swap, file_count = 0, dir_count = 0, sym_count = 0, dev_count = 0, fifo_count = 0;
+int bytes = 0, swap, file_count = 0, dir_count = 0, sym_count = 0,
+	dev_count = 0, fifo_count = 0;
 char *inode_table = NULL, *directory_table = NULL;
 struct hash_table_entry *inode_table_hash[65536], *directory_table_hash[65536];
 int fd;
@@ -80,8 +81,9 @@ char *fragment_data;
 char *file_data;
 char *data;
 unsigned int block_size;
-int lsonly = FALSE, info = FALSE;
+int lsonly = FALSE, info = FALSE, force = FALSE;
 char **created_inode;
+int root_process;
 
 #define CALCULATE_HASH(start)	(start & 0xffff)
 
@@ -109,7 +111,8 @@ int lookup_entry(struct hash_table_entry *hash_table[], int start)
 	int hash = CALCULATE_HASH(start);
 	struct hash_table_entry *hash_table_entry;
 
-	for(hash_table_entry = hash_table[hash]; hash_table_entry; hash_table_entry = hash_table_entry->next)
+	for(hash_table_entry = hash_table[hash]; hash_table_entry;
+				hash_table_entry = hash_table_entry->next)
 		if(hash_table_entry->start == start)
 			return hash_table_entry->bytes;
 
@@ -137,7 +140,8 @@ int read_bytes(long long byte, int bytes, char *buff)
 }
 
 
-int read_block(long long start, long long *next, char *block, squashfs_super_block *sBlk)
+int read_block(long long start, long long *next, char *block,
+squashfs_super_block *sBlk)
 {
 	unsigned short c_byte;
 	int offset = 2;
@@ -164,7 +168,8 @@ int read_block(long long start, long long *next, char *block, squashfs_super_blo
 		if(read_bytes(start + offset, c_byte, buffer) == FALSE)
 			goto failed;
 
-		if((res = uncompress((unsigned char *) block, &bytes, (const unsigned char *) buffer, c_byte)) != Z_OK) {
+		if((res = uncompress((unsigned char *) block, &bytes,
+		(const unsigned char *) buffer, c_byte)) != Z_OK) {
 			if(res == Z_MEM_ERROR)
 				ERROR("zlib::uncompress failed, not enough memory\n");
 			else if(res == Z_BUF_ERROR)
@@ -202,7 +207,8 @@ int read_data_block(long long start, unsigned int size, char *block)
 		if(read_bytes(start, c_byte, data) == FALSE)
 			return 0;
 
-		if((res = uncompress((unsigned char *) block, &bytes, (const unsigned char *) data, c_byte)) != Z_OK) {
+		if((res = uncompress((unsigned char *) block, &bytes,
+		(const unsigned char *) data, c_byte)) != Z_OK) {
 			if(res == Z_MEM_ERROR)
 				ERROR("zlib::uncompress failed, not enough memory\n");
 			else if(res == Z_BUF_ERROR)
@@ -222,13 +228,15 @@ int read_data_block(long long start, unsigned int size, char *block)
 }
 
 
-void uncompress_inode_table(long long start, long long end, squashfs_super_block *sBlk)
+void uncompress_inode_table(long long start, long long end,
+squashfs_super_block *sBlk)
 {
 	int size = 0, bytes = 0, res;
 
 	while(start < end) {
 		if((size - bytes < SQUASHFS_METADATA_SIZE) &&
-				((inode_table = realloc(inode_table, size += SQUASHFS_METADATA_SIZE)) == NULL))
+				((inode_table = realloc(inode_table, size +=
+				SQUASHFS_METADATA_SIZE)) == NULL))
 			EXIT_UNSQUASH("uncompress_inode_table: out of memory in realloc\n");
 		TRACE("uncompress_inode_table: reading block 0x%llx\n", start);
 		add_entry(inode_table_hash, start, bytes);
@@ -241,8 +249,8 @@ void uncompress_inode_table(long long start, long long end, squashfs_super_block
 }
 
 
-int set_attributes(char *pathname, unsigned int mode, unsigned int uid, unsigned int guid,
-unsigned int mtime, unsigned int set_mode)
+int set_attributes(char *pathname, unsigned int mode, unsigned int uid,
+unsigned int guid, unsigned int mtime, unsigned int set_mode)
 {
 	struct utimbuf times = { (time_t) mtime, (time_t) mtime };
 
@@ -251,12 +259,7 @@ unsigned int mtime, unsigned int set_mode)
 		return FALSE;
 	}
 
-	if(set_mode && chmod(pathname, (mode_t) mode) == -1) {
-		ERROR("set_attributes: failed to change mode %s, because %s\n", pathname, strerror(errno));
-		return FALSE;
-	}
-
-	if(geteuid() == 0) {
+	if(root_process) {
 		uid_t uid_value = (uid_t) uid_table[uid];
 		uid_t guid_value = guid == SQUASHFS_GUIDS ? uid_value : (uid_t) guid_table[guid];
 
@@ -264,6 +267,12 @@ unsigned int mtime, unsigned int set_mode)
 			ERROR("set_attributes: failed to change uid and gids on %s, because %s\n", pathname, strerror(errno));
 			return FALSE;
 		}
+	} else
+		mode &= ~07000;
+
+	if((set_mode || (mode & 07000)) && chmod(pathname, (mode_t) mode) == -1) {
+		ERROR("set_attributes: failed to change mode %s, because %s\n", pathname, strerror(errno));
+		return FALSE;
 	}
 
 	return TRUE;
@@ -277,9 +286,19 @@ void read_uids_guids(squashfs_super_block *sBlk)
 
 	guid_table = uid_table + sBlk->no_uids;
 
-	if(read_bytes(sBlk->uid_start, (sBlk->no_uids + sBlk->no_guids) * sizeof(unsigned int), (char *) uid_table) ==
-			FALSE)
-		EXIT_UNSQUASH("read_uids_guids: failed to read uid/gid table\n");
+	if(swap) {
+		unsigned int suid_table[sBlk->no_uids + sBlk->no_guids];
+
+		if(read_bytes(sBlk->uid_start, (sBlk->no_uids + sBlk->no_guids)
+				* sizeof(unsigned int), (char *) suid_table) ==
+				FALSE)
+			EXIT_UNSQUASH("read_uids_guids: failed to read uid/gid table\n");
+		SQUASHFS_SWAP_INTS(uid_table, suid_table, sBlk->no_uids + sBlk->no_guids);
+	} else
+		if(read_bytes(sBlk->uid_start, (sBlk->no_uids + sBlk->no_guids)
+				* sizeof(unsigned int), (char *) uid_table) ==
+				FALSE)
+			EXIT_UNSQUASH("read_uids_guids: failed to read uid/gid table\n");
 }
 
 
@@ -289,10 +308,13 @@ void read_fragment_table(squashfs_super_block *sBlk)
 	squashfs_fragment_index fragment_table_index[indexes];
 
 	TRACE("read_fragment_table: %d fragments, reading %d fragment indexes from 0x%llx\n", sBlk->fragments, indexes, sBlk->fragment_table_start);
+
 	if(sBlk->fragments == 0)
 		return;
 
-	if((fragment_table = (squashfs_fragment_entry *) malloc(sBlk->fragments * sizeof(squashfs_fragment_entry))) == NULL)
+	if((fragment_table = (squashfs_fragment_entry *)
+			malloc(sBlk->fragments *
+			sizeof(squashfs_fragment_entry))) == NULL)
 		EXIT_UNSQUASH("read_fragment_table: failed to allocate fragment table\n");
 
 	if(swap) {
@@ -304,7 +326,8 @@ void read_fragment_table(squashfs_super_block *sBlk)
 		read_bytes(sBlk->fragment_table_start, SQUASHFS_FRAGMENT_INDEX_BYTES(sBlk->fragments), (char *) fragment_table_index);
 
 	for(i = 0; i < indexes; i++) {
-		int length = read_block(fragment_table_index[i], NULL, ((char *) fragment_table) + (i * SQUASHFS_METADATA_SIZE), sBlk);
+		int length = read_block(fragment_table_index[i], NULL,
+		((char *) fragment_table) + (i * SQUASHFS_METADATA_SIZE), sBlk);
 		TRACE("Read fragment table block %d, from 0x%llx, length %d\n", i, fragment_table_index[i], length);
 	}
 
@@ -336,8 +359,9 @@ char *read_fragment(unsigned int fragment)
 }
 
 
-int write_file(char *pathname, unsigned int fragment, unsigned int frag_bytes, unsigned int offset,
-unsigned int blocks, long long start, char *block_ptr, unsigned int mode)
+int write_file(char *pathname, unsigned int fragment, unsigned int frag_bytes,
+unsigned int offset, unsigned int blocks, long long start, char *block_ptr,
+unsigned int mode)
 {
 	unsigned int file_fd, bytes, i;
 	unsigned int *block_list;
@@ -356,7 +380,7 @@ unsigned int blocks, long long start, char *block_ptr, unsigned int mode)
 	} else
 		memcpy(block_list, block_ptr, blocks * sizeof(unsigned int));
 
-	if((file_fd = open(pathname, O_CREAT | O_WRONLY, (mode_t) mode)) == -1) {
+	if((file_fd = open(pathname, O_CREAT | O_WRONLY | (force ? O_TRUNC : 0), (mode_t) mode & 0777)) == -1) {
 		ERROR("write_file: failed to create file %s, because %s\n", pathname,
 			strerror(errno));
 		free(block_list);
@@ -399,7 +423,8 @@ failure:
 }
 		
 
-int create_inode(char *pathname, unsigned int start_block, unsigned int offset, squashfs_super_block *sBlk)
+int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
+squashfs_super_block *sBlk)
 {
 	long long start = sBlk->inode_table_start + start_block;
 	squashfs_inode_header header;
@@ -423,6 +448,9 @@ int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
 
 	if(created_inode[header.base.inode_number - 1]) {
 		TRACE("create_inode: hard link\n");
+		if(force)
+			unlink(pathname);
+
 		if(link(created_inode[header.base.inode_number - 1], pathname) == -1) {
 			ERROR("create_inode: failed to create hardlink, because %s\n", strerror(errno));
 			return FALSE;
@@ -446,18 +474,21 @@ int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
 			} else
 				memcpy(inode, block_ptr, sizeof(*inode));
 
-			frag_bytes = inode->fragment == SQUASHFS_INVALID_FRAG ? 0 : inode->file_size % sBlk->block_size;
+			frag_bytes = inode->fragment == SQUASHFS_INVALID_FRAG ?
+				0 : inode->file_size % sBlk->block_size;
 			offset = inode->offset;
-			blocks = inode->fragment == SQUASHFS_INVALID_FRAG ? (inode->file_size
-				+ sBlk->block_size - 1) >> sBlk->block_log : inode->file_size >>
+			blocks = inode->fragment == SQUASHFS_INVALID_FRAG ?
+				(inode->file_size + sBlk->block_size - 1) >>
+				sBlk->block_log : inode->file_size >>
 				sBlk->block_log;
 			start = inode->start_block;
 
 			TRACE("create_inode: regular file, file_size %lld, blocks %d\n", inode->file_size, blocks);
 
-			if(write_file(pathname, inode->fragment, frag_bytes, offset, blocks, start,
-					block_ptr + sizeof(*inode), inode->mode)) {
-				set_attributes(pathname, inode->mode, inode->uid, inode->guid, inode->mtime, FALSE);
+			if(write_file(pathname, inode->fragment, frag_bytes,
+					offset, blocks, start, block_ptr +
+					sizeof(*inode), inode->mode)) {
+				set_attributes(pathname, inode->mode, inode->uid, inode->guid, inode->mtime, force);
 				file_count ++;
 			}
 			break;
@@ -476,18 +507,21 @@ int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
 			} else
 				memcpy(inode, block_ptr, sizeof(*inode));
 
-			frag_bytes = inode->fragment == SQUASHFS_INVALID_FRAG ? 0 : inode->file_size % sBlk->block_size;
+			frag_bytes = inode->fragment == SQUASHFS_INVALID_FRAG ?
+				0 : inode->file_size % sBlk->block_size;
 			offset = inode->offset;
-			blocks = inode->fragment == SQUASHFS_INVALID_FRAG ? (inode->file_size
-				+ sBlk->block_size - 1) >> sBlk->block_log : inode->file_size >>
+			blocks = inode->fragment == SQUASHFS_INVALID_FRAG ?
+				(inode->file_size + sBlk->block_size - 1) >>
+				sBlk->block_log : inode->file_size >>
 				sBlk->block_log;
 			start = inode->start_block;
 
 			TRACE("create_inode: regular file, file_size %lld, blocks %d\n", inode->file_size, blocks);
 
-			if(write_file(pathname, inode->fragment, frag_bytes, offset, blocks, start,
-					block_ptr + sizeof(*inode), inode->mode)) {
-				set_attributes(pathname, inode->mode, inode->uid, inode->guid, inode->mtime, FALSE);
+			if(write_file(pathname, inode->fragment, frag_bytes,
+					offset, blocks, start, block_ptr +
+					sizeof(*inode), inode->mode)) {
+				set_attributes(pathname, inode->mode, inode->uid, inode->guid, inode->mtime, force);
 				file_count ++;
 			}
 			break;
@@ -508,15 +542,20 @@ int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
 			strncpy(name, block_ptr + sizeof(squashfs_symlink_inode_header), inodep->symlink_size);
 			name[inodep->symlink_size] = '\0';
 
+			if(force)
+				unlink(pathname);
+
 			if(symlink(name, pathname) == -1) {
 				ERROR("create_inode: failed to create symlink %s, because %s\n", pathname,
 					strerror(errno));
 				break;
 			}
 
-			if(geteuid() == 0) {
+			if(root_process) {
 				uid_t uid_value = (uid_t) uid_table[inodep->uid];
-				uid_t guid_value = inodep->guid == SQUASHFS_GUIDS ? uid_value : (uid_t) guid_table[inodep->guid];
+				uid_t guid_value = inodep->guid ==
+					SQUASHFS_GUIDS ? uid_value : (uid_t)
+					guid_table[inodep->guid];
 
 				if(lchown(pathname, uid_value, guid_value) == -1)
 					ERROR("create_inode: failed to change uid and gids on %s, because %s\n", pathname, strerror(errno));
@@ -538,10 +577,14 @@ int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
 
 			TRACE("create_inode: dev, rdev 0x%x\n", inodep->rdev);
 
-			if(geteuid() == 0) {
-				if(mknod(pathname, inodep->inode_type == SQUASHFS_CHRDEV_TYPE ? S_IFCHR : S_IFBLK,
-							makedev((inodep->rdev >> 8) & 0xff, inodep->rdev & 0xff))
-							== -1) {
+			if(root_process) {
+				if(force)
+					unlink(pathname);
+
+				if(mknod(pathname, inodep->inode_type ==
+					SQUASHFS_CHRDEV_TYPE ?  S_IFCHR :
+					S_IFBLK, makedev((inodep->rdev >> 8)
+					& 0xff, inodep->rdev & 0xff)) == -1) {
 					ERROR("create_inode: failed to create %s device %s, because %s\n",
 						inodep->inode_type == SQUASHFS_CHRDEV_TYPE ? "character" : "block",
 						pathname, strerror(errno));
@@ -558,6 +601,9 @@ int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
 		case SQUASHFS_FIFO_TYPE:
 			TRACE("create_inode: fifo\n");
 
+			if(force)
+				unlink(pathname);
+
 			if(mknod(pathname, S_IFIFO, 0) == -1) {
 				ERROR("create_inode: failed to create fifo %s, because %s\n",
 					pathname, strerror(errno));
@@ -582,12 +628,15 @@ int create_inode(char *pathname, unsigned int start_block, unsigned int offset,
 }
 
 
-void uncompress_directory_table(long long start, long long end, squashfs_super_block *sBlk)
+void uncompress_directory_table(long long start, long long end,
+squashfs_super_block *sBlk)
 {
 	int bytes = 0, size = 0, res;
 
 	while(start < end) {
-		if(size - bytes < SQUASHFS_METADATA_SIZE && (directory_table = realloc(directory_table, size += SQUASHFS_METADATA_SIZE)) == NULL)
+		if(size - bytes < SQUASHFS_METADATA_SIZE && (directory_table =
+				realloc(directory_table, size +=
+				SQUASHFS_METADATA_SIZE)) == NULL)
 			EXIT_UNSQUASH("uncompress_directory_table: out of memory in realloc\n");
 		TRACE("uncompress_directory_table: reading block 0x%llx\n", start);
 		add_entry(directory_table_hash, start, bytes);
@@ -618,7 +667,8 @@ struct dir {
 };
 
 
-struct dir *squashfs_openddir(unsigned int block_start, unsigned int offset, squashfs_super_block *sBlk)
+struct dir *squashfs_openddir(unsigned int block_start, unsigned int offset,
+squashfs_super_block *sBlk)
 {
 	squashfs_dir_header dirh;
 	char buffer[sizeof(squashfs_dir_entry) + SQUASHFS_NAME_LEN + 1];
@@ -737,7 +787,8 @@ struct dir *squashfs_openddir(unsigned int block_start, unsigned int offset, squ
 }
 
 
-int squashfs_readdir(struct dir *dir, char **name, unsigned int *start_block, unsigned int *offset, unsigned int *type)
+int squashfs_readdir(struct dir *dir, char **name, unsigned int *start_block,
+unsigned int *offset, unsigned int *type)
 {
 	if(dir->cur_entry == dir->dir_count)
 		return FALSE;
@@ -759,18 +810,45 @@ void squashfs_closedir(struct dir *dir)
 }
 
 
-int dir_scan(char *parent_name, unsigned int start_block, unsigned int offset, squashfs_super_block *sBlk)
+char *get_component(char *target, char *targname)
+{
+	while(*target == '/')
+		*target ++;
+
+	while(*target != '/' && *target!= '\0')
+		*targname ++ = *target ++;
+
+	*targname = '\0';
+
+	return target;
+}
+
+
+int matches(char *targname, char *name)
+{
+	if(*targname == '\0' || strcmp(targname, name) == 0)
+		return TRUE;
+
+	return FALSE;
+}
+
+
+int dir_scan(char *parent_name, unsigned int start_block, unsigned int offset,
+squashfs_super_block *sBlk, char *target)
 {
 	struct dir *dir = squashfs_openddir(start_block, offset, sBlk);
 	unsigned int type;
 	char *name, pathname[1024];
+	char targname[1024];
+
+	target = get_component(target, targname);
 
 	if(dir == NULL) {
 		ERROR("dir_scan: Failed to read directory %s (%x:%x)\n", parent_name, start_block, offset);
 		return FALSE;
 	}
 
-	if(!lsonly && mkdir(parent_name, (mode_t) dir->mode) == -1) {
+	if(!lsonly && mkdir(parent_name, (mode_t) dir->mode) == -1 && (!force || errno != EEXIST)) {
 		ERROR("dir_scan: failed to open directory %s, because %s\n", parent_name, strerror(errno));
 		return FALSE;
 	}
@@ -778,19 +856,23 @@ int dir_scan(char *parent_name, unsigned int start_block, unsigned int offset, s
 	while(squashfs_readdir(dir, &name, &start_block, &offset, &type)) {
 		TRACE("dir_scan: name %s, start_block %d, offset %d, type %d\n", name, start_block, offset, type);
 
+
+		if(!matches(targname, name))
+			continue;
+
 		strcat(strcat(strcpy(pathname, parent_name), "/"), name);
 
 		if(lsonly || info)
 			printf("%s\n", pathname);
 
 		if(type == SQUASHFS_DIR_TYPE)
-			dir_scan(pathname, start_block, offset, sBlk);
+			dir_scan(pathname, start_block, offset, sBlk, target);
 		else
 			if(!lsonly)
 				create_inode(pathname, start_block, offset, sBlk);
 	}
 
-	!lsonly && set_attributes(parent_name, dir->mode, dir->uid, dir->guid, dir->mtime, TRUE);
+	!lsonly && set_attributes(parent_name, dir->mode, dir->uid, dir->guid, dir->mtime, force);
 
 	squashfs_closedir(dir);
 	dir_count ++;
@@ -848,8 +930,7 @@ int read_super(squashfs_super_block *sBlk, char *source)
 	TRACE("sBlk->inode_table_start 0x%llx\n", sBlk->inode_table_start);
 	TRACE("sBlk->directory_table_start 0x%llx\n", sBlk->directory_table_start);
 	TRACE("sBlk->uid_start 0x%llx\n", sBlk->uid_start);
-	TRACE("sBlk->fragment_table_start 0x%llx\n", sBlk->fragment_table_start);
-	TRACE("\n");
+	TRACE("sBlk->fragment_table_start 0x%llx\n\n", sBlk->fragment_table_start);
 
 	return TRUE;
 
@@ -859,7 +940,7 @@ failed_mount:
 
 
 #define VERSION() \
-	printf("unsquashfs version 1.0 (2006/03/15)\n");\
+	printf("unsquashfs version 1.2 (2006/08/19)\n");\
 	printf("copyright (C) 2006 Phillip Lougher <phillip@lougher.org.uk>\n\n"); \
     	printf("This program is free software; you can redistribute it and/or\n");\
 	printf("modify it under the terms of the GNU General Public License\n");\
@@ -874,36 +955,45 @@ int main(int argc, char *argv[])
 	squashfs_super_block sBlk;
 	char *dest = "squashfs-root";
 	int i, version = FALSE;
+	char *target = "";
 
+	if(root_process = geteuid() == 0)
+		umask(0);
+	
 	for(i = 1; i < argc; i++) {
 		if(*argv[i] != '-')
 			break;
-		if(strcmp(argv[i], "-version") == 0) {
+		if(strcmp(argv[i], "-version") == 0 || strcmp(argv[i], "-v") == 0) {
 			VERSION();
 			version = TRUE;
-		} else if(strcmp(argv[i], "-info") == 0)
+		} else if(strcmp(argv[i], "-info") == 0 || strcmp(argv[i], "-i") == 0)
 			info = TRUE;
-		else if(strcmp(argv[i], "-ls") == 0)
+		else if(strcmp(argv[i], "-ls") == 0 || strcmp(argv[i], "-l") == 0)
 			lsonly = TRUE;
-		else if(strcmp(argv[i], "-dest") == 0) {
+		else if(strcmp(argv[i], "-dest") == 0 || strcmp(argv[i], "-d") == 0) {
 			if(++i == argc)
 				goto options;
 			dest = argv[i];
-		}
+		} else if(strcmp(argv[i], "-force") == 0 || strcmp(argv[i], "-f") == 0)
+			force = TRUE;
 	}
 
 	if(i == argc) {
 		if(!version) {
 options:
-			ERROR("SYNTAX: %s [-ls | -dest] filesystem\n", argv[0]);
-			ERROR("\t-version\t\tprint version, licence and copyright information\n");
-			ERROR("\t-info\t\t\tprint files as they are unsquashed\n");
-			ERROR("\t-ls\t\t\tlist filesystem only\n");
-			ERROR("\t-dest <pathname>\tunsquash to <pathname>, default \"squashfs-root\"\n");
+			ERROR("SYNTAX: %s [options] filesystem [directory or file to extract]\n", argv[0]);
+			ERROR("\t-v[ersion]\t\tprint version, licence and copyright information\n");
+			ERROR("\t-i[nfo]\t\t\tprint files as they are unsquashed\n");
+			ERROR("\t-l[s]\t\t\tlist filesystem only\n");
+			ERROR("\t-d[est] <pathname>\tunsquash to <pathname>, default \"squashfs-root\"\n");
+			ERROR("\t-f[orce]\t\tif file already exists then overwrite\n");
 		}
 		exit(1);
 	}
 
+	if((i + 1) < argc)
+		target = argv[i + 1];
+
 	if((fd = open(argv[i], O_RDONLY)) == -1) {
 		ERROR("Could not open %s, because %s\n", argv[i], strerror(errno));
 		exit(1);
@@ -932,7 +1022,7 @@ options:
 	uncompress_inode_table(sBlk.inode_table_start, sBlk.directory_table_start, &sBlk);
 	uncompress_directory_table(sBlk.directory_table_start, sBlk.fragment_table_start, &sBlk);
 
-	dir_scan(dest, SQUASHFS_INODE_BLK(sBlk.root_inode), SQUASHFS_INODE_OFFSET(sBlk.root_inode), &sBlk);
+	dir_scan(dest, SQUASHFS_INODE_BLK(sBlk.root_inode), SQUASHFS_INODE_OFFSET(sBlk.root_inode), &sBlk, target);
 
 	if(!lsonly) {
 		printf("\n");
@@ -942,5 +1032,5 @@ options:
 		printf("created %d devices\n", dev_count);
 		printf("created %d fifos\n", fifo_count);
 	}
-
+	return 0;
 }
